#-*-coding:utf-8-*-

"""
petite collection de fonctions qui calculent les
coeffs de filtres BP récursifs et qui les appliquent.
"""

import numpy as NY
from samplerate import sr
from conversions08 import prony2freq

__author__ = "Copyright (c) René Bastian 2002-2006 <rbastian@free.fr>"
__date__ = "2003.08.11, 2004.05.22, 2006.02.13"


def _coeffsBPrecursif(pronycentral, deltaprony):
    """
    calcule les coeffs pour un filtre PB recursif
    centré sur 'pronycentral' avec atténuation de -3 db
    de 'pronycentral-deltaprony' à 'pronycentral+deltaprony'
    'pronycentral' : hauteur centrale en prony, float 
    'deltaprony' : largeur en prony, float
    """
    freqc = prony2freq(pronycentral)
    freq1 = prony2freq(pronycentral - deltaprony)
    freq2 = prony2freq(pronycentral + deltaprony)
    bande = float(freq2-freq1)/sr
    fc = freqc * 1.0/sr
    omega = 2 * NY.pi * fc
    cosomega = NY.cos(omega)
    R = 1.0 - 3 * bande    
    R2 = R * R
    try:
        K = (1.0 - 2 * R * cosomega + R2)/(2.0 - 2 * cosomega)
    except ZeroDivisionError:
        print "(2.0 - 2 * cosomega) =", (2.0 - 2 * cosomega), cosomega
        return
    a0 = 1.0 - K
    a1 = 2 * (K - R) * cosomega
    a2 = R2 - K
    b1 = 2 * R * cosomega
    b2 = -R2
    return [a0, a1, a2], [0.0, b1, b2]

def coeffsBPrecursif(pronycentral, deltaprony):
    """
    calcule les coeffs pour un filtre PB recursif
    centré sur 'pronycentral' avec atténuation de -3 db
    de 'pronycentral-deltaprony' à 'pronycentral+deltaprony'
    'pronycentral' : hauteur centrale en prony, float 
    'deltaprony' : largeur en prony, float
    """
    freqc = prony2freq(pronycentral)
    freq1 = prony2freq(pronycentral - deltaprony)
    freq2 = prony2freq(pronycentral + deltaprony)
    bande = float(freq2-freq1)/sr
    fc = freqc * 1.0/sr
    
    omega = 2.0 * NY.pi * fc
    cosomega = NY.cos(omega)
    
    R = 1.0 - 3 * bande    
    R2 = R * R
    try:
        K = (1.0 - 2 * R * cosomega + R2)/(2.0 - 2 * cosomega)
    except ZeroDivisionError:
        print "(2.0 - 2 * cosomega) =", (2.0 - 2 * cosomega), cosomega
        K = 1.0
    a0 = 1.0 - K
    a1 = 2 * (K - R) * cosomega
    a2 = R2 - K
    b1 = 2 * R * cosomega
    b2 = -R2
    return [a0, a1, a2], [0.0, b1, b2]

def passebanderecursif(freqcoupure, bandepassante):
    """
    identique à coeffsBPrecursif (mais hauteur = fréquences)
    calcule les coefficients (a0, a1, a2), (0.0, b1, b2)
    d'un filtre BP récursif caractérisé par
    sa fréquence centrale et sa bande passante
    'freqcoupure' : fréquence centrale
    'bandepassante' : bande passante
    """
    bande = bandepassante * 1.0/sr
    fc = freqcoupure * 1.0/sr
    omega = 2 * NY.pi * fc
    cosomega = NY.cos(omega)
    R = 1.0 - 3.0 * bande
    R2 = R * R
    K = (1.0 - 2 * R * cosomega + R2)/(2.0 - 2 * cosomega)
    a0 = 1.0 - K
    a1 = 2 * (K - R) * cosomega 
    a2 = R2 - K
    b1 = 2 * R * cosomega
    b2 = -R2
    return [a0, a1, a2], [0.0, b1, b2]

def appliBPrecursif(x, a, b):
    """
    x : signal entrant
    a, b : liste coeffs 
     = > b[0] == 0.0
    """
    x = NY.concatenate((NY.zeros(2, NY.float64), x))
    y = NY.zeros(len(x), NY.float64)
    for i in range(2, len(x)-2):
        
        y[i] = x[i] * a[0]+ \
             x[i-1] * a[1]+ \
             x[i-2] * a[2]+ \
             y[i-1] * b[1]+ \
             y[i-2] * b[2]
    return y

def filtreBPrecursif( signal, pronycentral, deltaprony):
    """
    calcule les coeffs a, b 
    applique le filtre au signal
    """
    a, b = coeffsBPrecursif(pronycentral, deltaprony)
    print a, b
    y = appliBPrecursif( signal, a, b)
    return y

def appliBPrecursifProny(signal, pronycentral, deltaprony):
    "idem que 'filtreBPrecursif'"
    return filtreBPrecursif( signal, pronycentral, deltaprony)
    

def coeffsBPrecursifVar(liste):
    """
    liste  =  [[pronycentral, deltaprony], [pronycentral, deltaprony], ...]
    """
    A, B = [], []
    for c in liste:
        a, b = coeffsBPrecursif(c[0], c[1])
        A.append(a)
        B.append(b)
    return A, B

def appliFiltreVar(x, A, B):
    """
    appliFiltreVar applique une liste de coeffs de filtres généraux d'ordre 2
    x : signal entrant
    A, B : liste coeffs a & b
     = > B[i][0] == 0.0
    Normalement, len(x)== len(A)== len(B)
    """
    d = min(len(x), len(A), len(B))
    x = NY.concatenate((NY.zeros(2, NY.float64), x))
    y = NY.zeros(len(x), NY.float64)
    
    for i in range(2, d): 
        y[i] = x[i] * A[i][0]+x[i-1] * A[i][1]+x[i-2] * A[i][2]+ \
                y[i-1] * B[i][1]+y[i-2] * B[i][2]
    return y

def appliBPrecursifPronyVar(signal, liste):
    """
    liste  =  [[pronycentral, deltaprony], [pronycentral, deltaprony], ...]
    """
    A, B = coeffsBPrecursifVar(liste)
    y = appliFiltreVar(signal, A, B)
    return y

def filtreBPrecursifPronyVar(signal, liste):    
    """
    liste  =  [[pronycentral, deltaprony], [pronycentral, deltaprony], ...]
    """
    A, B = coeffsBPrecursifVar(liste)
    y = appliFiltreVar(signal, A, B)
    return y


def filtrePBRecursifProny(signal, pronycoupure, pronybandepassante):
    """
    applique un filtre BP à un signal
    'signal' : le signal à filtrer
    'pronycoupure' : la hauteur en prony de la frequence centrale de
    la bande
    'pronybandepassante' : l'intervalle en prony de la bande passante
    """
    freqcoupure = prony2freq(pronycoupure)
    bandepassante = prony2freq(pronybandepassante)
    a, b = passebanderecursif(freqcoupure, bandepassante)
    
    return appliBPrecursif(signal, a, b)

def filtrePBRecursif(signal, freqcoupure, bandepassante):
    """
    application d'un filtre BP à un 'signal' 
    selon une fréquence de coupure et une bande passante.
    """
    a, b = passebanderecursif(freqcoupure, bandepassante)
    return appliBPrecursif(signal, a, b)


