#-*-coding:utf-8-*-

"""
guirlande.py fournit la classe Guirlande
"""


__author__ = "Copyright (c) 2006-09 René Bastian (Wissembourg, FRANCE)"
__date__ = "2006.08.17, 08.12.18, 09.02.02"

from profils import cubica, recta, curva, profil
import numpy as NY
from conversions08 import prony2freq
from origine import Origine


class Guirlande(Origine):
    """
    une classe pour faire des guirlandes de tons
    """
    def __init__(self, duree=None, dt=None, listeFF=None):
        """
        construit une liste de plateaux et de transitions (portamento)
        les valeurs retournées seront un NY.array de fréquences ;
        selon la méthode choisie, les descriptions
        peuvent être en prony ou en hertz.
        
        'dt'  durée du portamento

        Les paramètres suivants sont plus utiles dans les méthodes :
        'duree'  durée du plateau 
        'listeFF'  liste des hauteurs ;
        cette construction permet de faire une concatenation
        de guirlandes hétérogènes
        e = concatenate((g.fac(duree1, dt1, listeFF1), g.fac(duree2, dt2, listeFF2)))
        peut servir pour trilles, tremolos, ...
        """
        Origine.__init__(self)
        self.duree = duree
        self.dt = dt
        if listeFF == None:
            self.listeFF = []
        else:
            self.listeFF = listeFF
        try:
            self.listeMF = self._c(listeFF)
        except StandardError:
            self.listeMF = None

    def __call__(self, duree, listeFF):
        """
        retourne la guirlande décrite par 'listeFF'.
        'duree'  durée finale de la séquence
        'listeFF'  liste des hauteurs successives (en prony ou Hz)

        Le choix
        'prony'  si 'prony', alors 'listeFF' est exprimé en 'prony'.
        a été éliminé: normalement l'erreur s'entend.
        """
        self.duree = duree
        ff = listeFF[0]
        listeMF = [ff]
        for i in range(1, len(listeFF)):
            if ff[0] == recta:
                y0 = ff[3]
            elif ff[0] == cubica:
                y0 = ff[4]
            elif ff[0] == curva:
                y0 = ff[3]
            ff = listeFF[i]
            y1 = ff[2]
            mfb = [cubica, self.dt, y0, 0., y1, 0.]
            listeMF.append(mfb)
            listeMF.append(ff)
            if ff == listeFF[-1]:
                break
        return listeMF

    def fac(self, duree, dt, listeFF):
        """
        fait calculer le profil
        """
        self.duree = duree
        self.dt = dt
        self.listeFF = listeFF
        self.listeMF = self._c(listeFF)
        return profil(self.listeMF)
    
    def profil(self):
        """
        calcule le profil
        """
        try :
            profil(self.listeMF)
        except StandardError:
            return None
    
    def _c(self, listeFF):
        """
        'listeFF'  liste de hauteurs différentes
        Retourne une succession de
        plateau portamento plateau portamento ... plateau.
        """
        listeMF = []
        for i in range(len(listeFF)):
            ff0 = listeFF[i]        
            mfa = [recta, self.duree, ff0, ff0]        
            listeMF.append(mfa)
            try :
                ff1 = listeFF[i+1]
            except StandardError:
                break
            mfb = [recta, self.dt, ff0, ff1]
            listeMF.append(mfb)
        return listeMF


    def traitM(self, duree, dt, prony0, prony1, dprony):
        """retourne un trait allant de 'prony0' à 'prony1'
        par degrés 'dprony' avec des portamentos de durée 'dt'
        sous forme de profil pour modulation de fréquence (échelle prony)
        'prony0'  valeur initiale
        'prony1'  valeur finale
        'dprony'  incrément (+ ou -)
        """
        Freqs = prony2freq(NY.arange(prony0, prony1, dprony))
        return self.fac(duree, dt, Freqs)

    def traitF(self, duree, dt, freq0, freq1, dfreq):
        """
        retourne un profil pour modulation de fréquence
        comme traitM mais l'échelle est en fréquences.        
        """
        Freqs = NY.arange(freq0, freq1, dfreq)
        return self.fac(duree, dt, Freqs)

    def traitL(self, duree, dt, Liste):
        
        """retourne un profil pour modulation de fréquence (échelle prony)
        'Liste'  liste de valeurs prony
        """
        Freqs = prony2freq(NY.array(Liste))
        return self.fac(duree, dt, Freqs)

    def listeprony2guirlande(self, duree, dt, Liste):
        """traduit une liste prony en guirlande de freqs
        """
        return self.traitL(duree, dt, Liste)

    def listefreqs2guirlande(self, duree, dt, listeFreqs):
        """traduit une liste freqs en guirlande de freqs
        """
        return self.traitG(duree, dt, listeFreqs)

    def traitG(self, duree, dt, listeFreqs):
        """
        comme traitL mais avec avec
        une liste de fréquences.
        """
        
        return self.fac(duree, dt, listeFreqs)
    
    def liste(self, listeFF=None):
        """ à quoi bon ???"""
        if listeFF == None:
            return profil(self.listeMF)
        else:
            self.listeMF = self._c(listeFF)
            return profil(self.listeMF)



