#-*-coding:utf-8-*-

"""
module générateur  d'un timbre de percussion
"""
import numpy as NY
import decibel
import profils as _p
import ondes as _o
import canal
import serie
from origine import Origine


__author__ = "Copyright (c) 2005-07 René Bastian (Wissembourg, FRANCE)"
__date__ = "2008.07.17"


class PercuEthnique3(Origine):
    """un timbre de percussion"""
    def __init__(self, info=True, ncoeffs=200, dampli=None, 
                 couple=(2458, 4731)):
        """
        'ncoeffs' : nombre de coeffs des filtres
        'couple' : semence du générateur sériel
        Pour utiliser un 'db' variable, faire la mutation
        'db' -> 'ampli' par vecteur complet.
        """
        Origine.__init__(self)
        self.alea = serie.Serie002(couple[0], couple[1])
        self.ncoeffs = ncoeffs
        self.rf = _o.RecursifFiltre("pb", self.ncoeffs)
        if dampli != None:
            self.dampli = dampli
            self.db = decibel.Decibel()
        else:
            self.dampli = dampli
            self.db = None
        if info:
            self.info = True
            self.listeInfo = []
        else:
            self.info = False

    def evt(self, nombre, pEcart, pProny, pCRecurs, pAmplis, pPanos):
        """
        suite d'évènements de 'percu' décrits par des profils-listes
        'nombre' : nombre d'évènements
        'pEcart' : profil des écarts, 
        'pProny' : profil des hauteurs
        'pAmplis' : profil des amplis 
        'pPanos' : profil des panos
        'pCRecurs' : évolution du coeff de récursivité
        """
        cg = canal.CanalMono()
        cd = canal.CanalMono()
        pt = 0.0
        Ecarts = _p.ProfilBase(pEcart).samples(nombre)
        Pronys = _p.ProfilBase(pProny).samples(nombre)
        Amplis = _p.ProfilBase(pAmplis).samples(nombre)
        Panos = _p.ProfilBase(pPanos).samples(nombre)
        
        CRecurs = _p.ProfilBase(pCRecurs).samples(nombre)
        for ecart, Prony, ampli, r, pano in \
                zip(Ecarts, Pronys, Amplis, CRecurs, Panos):
            w = self.rf.amortissant(Prony, r)                
            w *= ampli
            wg, wd = w * pano, w * (1.0 - pano)
            cg.addT(pt, wg)
            cd.addT(pt, wd)
            pt += ecart
        if self.info:
            self._autoinfo([Ecarts, Pronys, Amplis, CRecurs])
        return cg.ex(), cd.ex()

    def _autoinfo(self, listeP):
        """collecte les données dans une liste, qui peut servir
        à une édition graphique.
        """
        X = NY.array(listeP)
        X = X.transpose()
        self.listeInfo.append(X)

    def evtMono(self, listeP):
        """produit une suite d'évènements mono.
        Les params doivent être des listes-profils dans l'ordre:
          nombre, PEcart, PProny, PCRecurs, PAmplis.
        'listeP' : profils-listes
        si tous les 'ecart == 0.0', c'est aussi une belle percussion
        """    
        c = canal.CanalMono()
        pt = 0.0
        for p in listeP:
            nombre, PEcart, PProny, PCRecurs, PAmplis = p
            Ecarts = _p.ProfilBase(PEcart).samples(nombre)
            Pronys = _p.ProfilBase(PProny).samples(nombre)
            Amplis = _p.ProfilBase(PAmplis).samples(nombre)
            
            CRecurs = _p.ProfilBase(PCRecurs).samples(nombre)
            for ecart, Prony, ampli, r in zip(Ecarts, Pronys, Amplis, CRecurs):
                w = self.rf.amortissant(Prony, r)                
                w *= ampli
                c.addT(pt, w)                
                pt += ecart
            if self.info:
                self._autoinfo([Ecarts, Pronys, Amplis, CRecurs])
        return c.ex()
        
    def evtPano(self, listeP):
        """produit une suite d'évènements stéréo.
        Les params doivent être dans l'ordre:
          nombre, PEcart, PProny, PCRecurs, PAmplis, PPanos
        voir 'evtMono'
        """            
        cg = canal.CanalMono()
        cd = canal.CanalMono()
        pt = 0.0
        for p in listeP:
            nombre, PEcart, PProny, PCRecurs, PAmplis, PPanos = p
            Ecarts = _p.ProfilBase(PEcart).samples(nombre)
            Pronys = _p.ProfilBase(PProny).samples(nombre)
            Amplis = _p.ProfilBase(PAmplis).samples(nombre)
            CRecurs = _p.ProfilBase(PCRecurs).samples(nombre)
            Panos = _p.ProfilBase(PPanos).samples(nombre)
            for ecart, Prony, ampli, r, pano in \
                    zip(Ecarts, Pronys, Amplis, CRecurs, Panos):
                w = self.rf.amortissant(Prony, r)
                w *= ampli
                wg = w * pano
                wd = w * (1. - pano)
                cg.addT(pt, wg)
                cd.addT(pt, wd)
                pt += ecart
            if self.info:
                self._autoinfo([Ecarts, Pronys, Amplis, CRecurs, Panos])
        return cg.ex(), cd.ex()

    def close(self):
        "si 'info', fournit la liste des infos recueillies."
        if self.info:
            return self.listeInfo
        else:
            return None


