#-*-coding:utf-8-*-

"""
fournit la classe Rale"
"""
from conversions08 import prony2freq
from origine import Origine
import numpy as NY
from serie import Serie002
import canal08 as canal
import decibel
from profilbase import ProfilBase
from profils import recta, rectaf, curva, profil, profiltuple
from ondepartable import ondeTableMF


__author__ = "Copyright (c) 2006 René Bastian (Wissembourg, FRANCE)"
__date__ = "2006.11.23, 06.09.23, 07.11.12, 08.09.09"


class Rale(Origine):
    """
    La classe 'Rale' construit un timbre rugueux
    - par balayage d'une table 'Table'
    - et application d'une enveloppe 'Enveloppe'
    Le profil 'Amorti' n'est utilisé que dans 'evtR'.
    Lors de l'appel, il faut indiquer le nombre 'nombre' de
    superpositions. Ces superpositions donnent une densité
    au son.
    Le son peut être ultérieurement réverbéré en utilisant
    la classe 'Reverberation' et en mélangeant une proportion
    variable entre son direct et son réverbéré.

    Si l'intervalle 'pronys' est petit (<=1.5), le son
    est très fluide (pas du tout râlant). Si cet intervalle
    est de l'ordre de 3. ou 4. on peut obtenir des sons
    très mélodieux.

    Il faudrait donc essayer de moduler cet intervalle.
    """
    def __init__(self, alea, Table, Enveloppe, Amorti=None):
        """
        Amorti = [[curva, duree, 4.0, 0.0, 0.2 ]]
        """
        Origine.__init__(self)
        self.alea = alea
        self.env = ProfilBase(Enveloppe)
        self.table = ProfilBase(Table).temps(1.0)
        if Amorti != None:
            self.amorti = ProfilBase(Amorti)
        else:
            self.amorti = None
        self.db = decibel.Decibel()

    def __call__(self, duree, nombre, pronys, amps):
        """
        'duree' : durée de l'évènement
        'nombre' : nombre de sous-évènements
        'pronys' : plage de choix des pronys
        'amps' : plage de choix des amplis.
        """
        c = canal.CanalMono()
        while c.ncompte < nombre:
            prony0 = self.alea.uniform(pronys[0], pronys[1])
            prony1 = self.alea.uniform(pronys[0], pronys[1])
            MF = [[recta, duree, prony0, prony1]] 
            mf = profil(MF)
            mf = prony2freq(mf)            
            w = ondeTableMF(self.table, mf)
            n = len(w)
            env = self.env.samples(n)            
            w *= env
            amp = self.alea.uniform(amps[0], amps[1])
            amp = self.db.db2x(amp)
            w *= amp
            c.addT(0.0, w)            
        return c.ex()

    def evtmf(self, duree, nombre, MF, amps, debug=False):
        """
        duree : durée du signal à calculer
        nombre : nombre de superpositions de sons
        MF : profil-liste avec des tuples pour variation sérielle
        amps : intervalle de variation de l'amplitude.
        """        
        c = canal.CanalMono()
        while c.ncompte < nombre:
            NMF = profiltuple(MF, self.alea, liste=True)
            if debug: 
                print "*", NMF
            mf = ProfilBase(NMF).temps(duree)
            mf = prony2freq(mf)            
            w = ondeTableMF(self.table, mf)
            n = len(w)
            env = self.env.samples(n)            
            w *= env
            amp = self.alea.uniform(amps[0], amps[1])
            amp = self.db.db2x(amp)
            w *= amp
            c.addT(0.0, w)            
        return c.ex()
    
    def evt(self, duree, nombre, pronys, amps):
        "comme __call__"
        return self.__call__(duree, nombre, pronys, amps)


if __name__ == "__main__":
    import live
    import sys

    audio = live.Audio()
    
    def test1():
        "pour tester"
        alea = Serie002(97531, 13567)
        Table = [[recta, 1.0, 0.0, 1.0],
                 [recta, 2.0, 1.0, -1.0],
                 [recta, 3.0, -1.0, 0.0]]
        Enveloppe = [[rectaf, 0.02, 0.0, 1.0],
                     [recta, 1.0, 1.0, 0.0]]
        Amorti = [[curva, 1.0, 4.0, 0.0, 0.2 ]]    
        cri = Rale(alea, Table, Enveloppe, Amorti)
        duree, nombre, pronys, amps = 5., 20, (60., 66.), (1.0, 8.0)
    
        M = NY.arange(24.0, 120.0, 6.0)
        format = "duree %7.3f n %3i pronys (%6.2f %6.2f) ampli (%6.2f %6.2f)"
        for m in M:
            pronys = (m, m + 6.0)
            print format % (duree, nombre, pronys[0], pronys[1], \
                                amps[0], amps[1])
            w = cri(duree, nombre, pronys, amps)
            audio.play(w, w)

    def test2():
        "produit un son dense"
        alea = Serie002(97531, 13567)
        Table = [[recta, 1.0, 0.0, 1.0],
                 [recta, 2.0, 1.0, -1.0],
                 [recta, 3.0, -1.0, 0.0]]
        Enveloppe = [[rectaf, 0.02, 0.0, 1.0],
                     [recta, 1.0, 1.0, 0.0]]
        Amorti = [[curva, 1.0, 4.0, 0.0, 0.2 ]]    
        rale = Rale(alea, Table, Enveloppe, Amorti)
        print rale
        duree, nombre, amps = 10., 20, (1.0, 8.0)
        MF = [[recta, 10., 36., 108.]]
        w = rale.evtmf(duree, nombre, MF, amps)
        
        duree = 4.
        MF = [[recta, 10., 66., 67.]]
        w = rale.evtmf(duree, nombre, MF, amps)
        audio.play(w, w)

    def test3():
        "pour tester"
        alea = Serie002(97531, 13567)
        Table = [[recta, 1.0, 0.0, 1.0],
                 [recta, 2.0, 1.0, -1.0],
                 [recta, 3.0, -1.0, 0.0]]
        Enveloppe = [[rectaf, 0.02, 0.0, 1.0],
                     [recta, 1.0, 1.0, 0.0]]
        Amorti = [[curva, 1.0, 4.0, 0.0, 0.2 ]]    
        rale = Rale(alea, Table, Enveloppe, Amorti)

        duree, nombre, amps = 2., 20, (1.0, 8.0)
        MF = [[recta, 10., 36., 108.]]
        w = rale.evtmf(duree, nombre, MF, amps)
        
        
        c = canal.CanalMono(duree)
        MF = [[recta, 10., 66., 66.]]
        w = rale.evtmf(duree, nombre, MF, amps)
        c.addT(0.0, w)

        MF = [[recta, 10., 70., 70.]]
        w = rale.evtmf(duree, nombre, MF, amps)
        c.addT(0.0, w)

        MF = [[recta, 10., 73., 73.]]
        w = rale.evtmf(duree, nombre, MF, amps)
        c.addT(0.0, w)
        audio.play(c.ex(), c.ex())

    def test4():
        "pour tester"
        pass
    
    try:
        ntest = int(sys.argv[-1])
    except ValueError:
        ntest = int(raw_input("Quel test ? "))
    print "Test", ntest

    if ntest == 1:
        test1()
    elif ntest == 2:
        test2()
    elif ntest == 3:
        test3()
    elif ntest == 4:
        test4()



